from celery import current_app as app

from .create_accredible_group import CreateAccredibleGroupTask  # noqa
from .create_accredible_consultant_role_group import CreateAccredibleConsultantRoleGroupTask  # noqa
from .create_accredible_simple_credential import CreateAccredibleSimpleCredentialTask  # noqa
from .create_accredible_group_credential import CreateAccredibleGroupCredentialTask  # noqa
from .delete_accredible_group import DeleteAccredibleGroupTask  # noqa
from .update_accredible_group import UpdateAccredibleGroupTask  # noqa

app.tasks.register(CreateAccredibleGroupTask())
app.tasks.register(CreateAccredibleConsultantRoleGroupTask())
app.tasks.register(CreateAccredibleSimpleCredentialTask())
app.tasks.register(CreateAccredibleGroupCredentialTask())
app.tasks.register(DeleteAccredibleGroupTask())
app.tasks.register(UpdateAccredibleGroupTask())



from ..signals_define import (
    create_certification_group,
    create_certification_credential,
    create_certification_group_credential,
    delete_certification_group,
    update_certification_group)

from .create_certification import (
    create_accredible_handler,
    create_credential_accredible_handler,
    create_group_and_credentials_handler,
    delete_accredible_handler,
    update_certification_group_handler)


def setup_signals():
    create_certification_group.connect(create_accredible_handler)
    delete_certification_group.connect(delete_accredible_handler)
    create_certification_credential.connect(create_credential_accredible_handler)
    create_certification_group_credential.connect(
        create_group_and_credentials_handler)
    update_certification_group.connect(
        update_certification_group_handler)



from ..models import CertificationGroup


def create_accredible_handler(signal, sender, *args, **kwargs):
    CertificationGroup.objects.create_group(
        **kwargs)


def delete_accredible_handler(instance, signal, sender, *args, **kwargs):
    group = instance.credentials.first()
    if group:
        CertificationGroup.objects.remove_group(group)


def create_credential_accredible_handler(certification_group, user_from, certification_credential, signal, sender, *args, **kwargs):
    CertificationGroup.objects.release_simple_credential(
        user_from=user_from,
        certification_group=certification_group,
        certification_credential=certification_credential)


def create_group_and_credentials_handler(user_from, course_name, issued_on, signal, sender, *args, **kwargs):
    CertificationGroup.objects.create_group_and_credentials(
        user_from=user_from,
        course_name=course_name,
        issued_on=issued_on,
        **kwargs)


def update_certification_group_handler(instance, user_from, signal, sender, *args, **kwargs):
    CertificationGroup.objects.update_group(
        user_from=user_from,
        certification_group=instance,
        **kwargs)



from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from model_utils.models import TimeStampedModel

from ..conf import settings
from ..manager import CertificationGroupManager


class CertificationGroup(TimeStampedModel):
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True, null=True,
        on_delete=models.deletion.CASCADE,
        related_name='certification_certificationgroup_related',
    )
    name = models.CharField(max_length=200)
    description = models.TextField(null=True)
    accredible_id = models.IntegerField(blank=True, null=True)
    design_id = models.IntegerField(blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        null=True,
    )
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')
    _type = models.CharField(
        max_length=100,
        choices=settings.CERTIFICATION_CH_GROUP_CH_TYPE)
    status = models.CharField(
        max_length=1,
        choices=settings.CERTIFICATION_CH_GROUP_STATUS,
        default=settings.CERTIFICATION_CH_GROUP_STATUS_DEFAULT)
    objects = CertificationGroupManager()

    def __str__(self):
        return self.name

    @property
    def instructor_name(self):
        instructor = None
        if self._type == settings.CERTIFICATION_CH_GROUP_WORKSHOP:
            instructor = self.content_object.created_by.get_full_name()
        return instructor
